@@ -41,7 +41,7 @@ replacements = {'prefix': 'SSH',
 cstr.set_fg_regex(pregex, **replacements)
 
 cstr.set_variables_infile(SSH='sea surface height')
-cstr.set_scan_in_file(scanlib.nc.scan_in_file, 'lat', 'lon', 'time')
+cstr.set_scan_in_file(scanlib.nc.scan_dims, 'lat', 'lon', 'time')
 
 cstr.set_scan_coords_attributes(scanlib.nc.scan_units, 'time')
 cstr.set_scan_general_attributes(scanlib.nc.scan_infos)
@@ -61,7 +61,7 @@ replacements = {'prefix': 'SSH',
 cstr.set_fg_regex(pregex, **replacements)
 
 cstr.set_variables_infile(SST='sst')
-cstr.set_scan_in_file(scanlib.nc.scan_in_file, 'lon', 'lat')
+cstr.set_scan_in_file(scanlib.nc.scan_dims, 'lon', 'lat')
 cstr.set_scan_filename(scanlib.get_date_from_matches, 'time', only_value=True)
 
 cstr.set_scan_general_attributes(scanlib.nc.scan_infos)
