@@ -7,69 +7,59 @@
 
 
 import logging
-from typing import Any, Dict, List, Union
+from typing import Any, Dict, List, Tuple, Type, Union, TYPE_CHECKING
 
-import numpy as np
-
-from tomate.accessor import Accessor
 from tomate.coordinates.coord import Coord
-from tomate.custom_types import KeyLike, KeyLikeInt, KeyLikeValue
-from tomate.filegroup.filegroup_load import FilegroupLoad
+from tomate.coordinates.variables import Variables
+from tomate.custom_types import Array, KeyLike, KeyLikeValue
 from tomate.keys.keyring import Keyring
 from tomate.scope import Scope
+from tomate.variable_base import Variable
 from tomate.variables_info import VariablesInfo
 
+if TYPE_CHECKING:
+    from tomate.db_types.data_disk import DataDisk
+
 
 log = logging.getLogger(__name__)
 
 
 class DataBase():
-    r"""Encapsulate data array and info about the variables.
+    r"""Encapsulate all elements of a database.
 
-    The data itself is stored in the data attribute.
-    The data can be conveniently accessed using the `view` method.
+    The DataBase object contains the scopes for available, loaded, and selected
+    data. It is the primary object to modify thoses scope (by loading or
+    selecting data for instance). Many methods use the current scope by default,
+    *ie* the loaded scope if data is loaded, available scope otherwise.
+    Coordinates of the current scope are available as attributes.
 
-    The data consists in multiple variables varying along
-    multiple coordinates.
-    An ensemble of coordinates and variables makes a `Scope`.
-    The data object has three different scopes:
-    \*avail: all data available on disk
-    \*loaded: part of that data that is loaded in memory
-    \*selected: part of data selected
-
-    Coordinates objects, the list of variables, the shape
-    of data, and other attributes of the different scopes objects,
-    are directly accessible from the data object.
-    If data has been loaded, the 'loaded' scope is used,
-    otherwise the 'available' scope is used.
-
-    Data and coordinates can be accessed as items:
-    `Data[name of variable]`.
+    Tha DataBase object also contains one or more Variable object. Those are
+    accessible as items with `Data[name of variable]`. Data can be retrieved
+    directly with the view method, eventually for multiple variable at once.
 
     See :doc:`../data` for more information.
 
-    :param vi: Information on the variables and data.
-    :param dims: Dimensions, (ie subclasses of Coord)
-        in the order the data should be kept.
-        This includes variables.
-        They are linked (not copied) to the available
-        scope.
-
-    :attr dims: List[str]: Dimensions names,
-        in the order the data is kept in the array.
+    :param dims: Dimensions, (ie subclasses of Coord). Variables can be omitted.
+        The dimensions will be stored in various scopes in the order they
+        are supplied. This same order is used in many methods when specifying
+        keys. They are linked (not copied) to the available scope.
+    :param vi: [opt] Information on the variables and data.
+        If None, one is created.
 
+    :attr dims: List[str]: Dimensions names in order.
     :attr vi: VariablesInfo: Information on the variables and data.
-    :attr data: np.ndarray or subclass: Data array if loaded, None otherwise.
     :attr avail: Scope: Scope of available data (on disk).
     :attr loaded: Scope: Scope of loaded data.
     :attr selected: Scope: Scope of selected data.
+    :attr variables: Dict[Variable]: Variable objects
+    :attr var_disk: set: Variables on disk.
     """
 
-    acs = Accessor  #: Accessor class (or subclass) to use to access the data.
-
     def __init__(self, dims: List[Coord],
                  vi: VariablesInfo = None):
 
+        if not any(d.name == 'var' for d in dims):
+            dims.insert(0, Variables([]))
         self.dims = [c.name for c in dims]
 
         self.avail = Scope(dims=dims, copy=False)
@@ -85,7 +75,8 @@ class DataBase():
             vi = VariablesInfo()
         self.vi = vi
 
-        self.data = None
+        self.var_disk = set()
+        self.variables = {}
 
     @property
     def coords(self) -> List[str]:
@@ -96,21 +87,32 @@ class DataBase():
         s = ["Data object"]
 
         s.append("Class: {}, Bases: {} ".format(self.__class__.__name__,
-                                                ', '.join(self.bases.values())))
+                                                ', '.join(self.bases.keys())))
         s.append('')
-        s.append("Data available: \n{}".format(str(self.avail)))
+        if self.variables:
+            s.append('Variables:')
+            for v in self.variables.values():
+                z = v.name
+                if not type(v) is Variable:
+                    z += ' ({}) '.format(type(v).__name__)
+                z += ' [{}] '.format(', '.join(v.dims))
+                if v.is_loaded():
+                    z += ', (loaded)'
+                s.append(z)
+            s.append('')
+        s.append("Data available: \n{}".format(repr(self.avail)))
         s.append('')
 
-        if self.data is None:
+        if self.loaded.is_empty():
             s.append('Data not loaded')
         else:
-            s.append('Data loaded: \n{}'.format(str(self.loaded)))
+            s.append('Data loaded: \n{}'.format(repr(self.loaded)))
         s.append('')
 
         if self.selected.is_empty():
             s.append('No data selected')
         else:
-            s.append('Data selected: \n{}'.format(str(self.selected)))
+            s.append('Data selected: \n{}'.format(repr(self.selected)))
         s.append('')
 
         return '\n'.join(s)
@@ -129,59 +131,32 @@ class DataBase():
                for c in bases}
         return out
 
-    def get_filegroup(self, key: Union[int, str]) -> FilegroupLoad:
-        """Get filegroup by index or name."""
-        if isinstance(key, int):
-            return self.filegroups[key]
-        if isinstance(key, str):
-            fgs = [fg for fg in self.filegroups
-                   if fg.name == key]
-            if len(fgs) == 0:
-                raise KeyError(f"No filegroup with name {key}")
-            if len(fgs) > 1:
-                raise IndexError(f"More than one filegroup with name {key}")
-            return fgs[0]
-        raise TypeError("Key must be filegroup index or name (is {})"
-                        .format(type(key)))
-
     def check_loaded(self):
-        """Check if data is loaded.
+        """Raises if data is not loaded.
 
         :raises RuntimeError: If the data is not loaded.
         """
-        if self.data is None:
+        if self.loaded.is_empty():
             raise RuntimeError("Data not loaded.")
 
-    def __getitem__(self, key: str) -> np.ndarray:
-        """Return a coordinate, or data for a variable.
-
-        If y is a variable name, return the corresponding data slice.
-
-        :param y: Coordinate or variable name or index.
-
-        :raises KeyError: If key is not a coordinate or variable.
-        """
+    def __getitem__(self, key: KeyLike) -> Variable:
+        """Return a variable object. """
         if isinstance(key, str):
-            if key in self.loaded:
-                key = self.idx(key)
-                return self.data[key]
-            raise KeyError(f"Variable '{key}' is not loaded.")
+            return self.variables[key]
         raise TypeError("Key must be a str.")
 
-    def __setitem__(self, key: str, value: np.ndarray):
+    def __setitem__(self, key: str, value: Array):
         """Assign data to a variable.
 
-        Wrapper around set_data
+        Wrapper around Variable.set_data
         """
-        self.set_data(key, value)
+        self.variables[key].set_data(value, Keyring())
 
-    def __getattribute__(self, name):
+    def __getattribute__(self, name: str):
         """Get attribute.
 
         If `name` is a coordinate name, return coordinate from
         current scope.
-        If `name` is 'var', return list of variable from
-        current scope.
         """
         if name in super().__getattribute__('dims'):
             if not self.loaded.is_empty():
@@ -199,7 +174,11 @@ class DataBase():
         return self.avail
 
     def get_scope(self, scope: Union[str, Scope]) -> Scope:
-        """Get scope by name."""
+        """Get scope by name.
+
+        :param scope: Can be {'avail', 'loaded', 'selected', 'current'},
+            or a Scope object, which will simply be returned.
+        """
         if isinstance(scope, str):
             scope = {'avail': self.avail,
                      'loaded': self.loaded,
@@ -207,47 +186,89 @@ class DataBase():
                      'current': self.scope}[scope]
         return scope
 
-    def idx(self, variable: Union[str, int]) -> int:
-        """Index of variables in the data array.
-
-        Wrapper around loaded Scope.idx()
-        """
-        return self.loaded.idx(variable)
-
-    def view(self, *keys: KeyLike,
-             keyring: Keyring = None, **kw_keys: KeyLike) -> np.ndarray:
+    def view(self, *keys: KeyLike, keyring: Keyring = None,
+             stack: str = None, order: List[str] = None,
+             **kw_keys: KeyLike) -> Union[Array, Tuple[Array]]:
         """Returns a subset of loaded data.
 
         Keys act on loaded scope.
         If a key is an integer, the corresponding dimension in the
         array will be squeezed.
+        If multiple variables are asked, a tuple containing each variable
+        data is returned. Multiple variables can also be stacked in the
+        same array. Data dimensions can be reordered
 
         :param keyring: [opt] Keyring specifying parts of dimensions to take.
         :param keys: [opt] Keys specifying parts of dimensions to take, in
-            the order dimensions are stored. Take precedence over `keyring`.
+            the order dimensions are stored in. Take precedence over `keyring`.
         :param kw_keys: [opt] Argument name is dimension name. Takes precedence
             over `keys`.
+        :param stack: [opt] Concatenate different variables into one array.
+            If is True, concatenate variables if they all have the same datatype
+            and dimensions. If equal to 'force', will concatenate even if
+            datatypes are different. Order of concatenation follow the variable
+            key. The first variable accessor is used for concatenation.
+        :param order: [opt] Reorder data dimensions. If of length 2,
+            the two dimensions will be swapped. Otherwise, '`order`'
+            must contain all the dimensions of the variable.
+            When stacking data, the order must contain all dimensions,
+            including 'var'. Squeezed dimensions are not taken into account.
 
         :returns: Subset of data.
+        :raises RuntimeError: If the user ask for an impossible stack
         """
-        self.check_loaded()
-
         kw_keys = self.get_kw_keys(*keys, **kw_keys)
-        keyring = Keyring.get_default(keyring, **kw_keys, variables=self.loaded.var)
+        keyring = Keyring.get_default(keyring, **kw_keys)
         keyring.make_full(self.dims)
         keyring.make_total()
-        keyring.simplify()
-        keyring.sort_by(self.dims)
-        log.debug('Taking keys in data: %s', keyring.print())
-        return self.acs.take(keyring, self.data)
+        keyring.make_idx_str(var=self.loaded.var)
 
-    def view_by_value(self, *keys: KeyLikeInt,
-                      by_day: bool = False,
-                      **kw_keys: KeyLike) -> np.ndarray:
+        variables = [self.variables[var] for var in keyring['var']]
+        dims = [[d for d in self.variables[var].dims
+                 if d in keyring.get_non_zeros()]
+                for var in keyring['var']]
+
+        if stack and len(variables) > 1:
+            if not all(v in self.loaded.var for v in keyring['var']):
+                raise RuntimeError("Cannot stack variables"
+                                   " (variable not loaded)")
+            if (stack != 'force'
+                and not all(v.datatype == variables[0].datatype
+                            for v in variables[1:])):
+                raise RuntimeError("Cannot stack variables"
+                                   " (different datatypes)")
+            if (order is None and not all(d == dims[0] for d in dims[1:])):
+                raise RuntimeError("Cannot stack variables (different "
+                                   "dimensions or order not specified)")
+            if (order is not None and not all(set(d) == set(dims[0])
+                                              for d in dims[1:])):
+                raise RuntimeError("Cannot stack variables"
+                                   " (different dimensions)")
+
+        if order is not None:
+            order_novar = [d for d in order if d != 'var']
+        else:
+            order_novar = None
+
+        out = tuple([var.view(keyring=keyring, order=order_novar)
+                     for var in variables])
+        if stack:
+            if order is not None:
+                axis_stack = order.index('var')
+            else:
+                axis_stack = self.dims.index('var')
+            out = variables[0].acs.stack(out, axis=axis_stack)
+        elif keyring['var'].size == 0:
+            out = out[0]
+
+        return out
+
+    def view_by_value(self, *keys: KeyLike, by_day: bool = False,
+                      stack: Union[str, bool] = None, order: List[str] = None,
+                      **kw_keys: KeyLike) -> Union[Array, Tuple[Array]]:
         """Returns a subset of loaded data.
 
-        Arguments work similarly as
-        :func:`DataDisk.load_by_value
+        Arguments work similarly as :func:`DataDisk.load_by_value
         <tomate.db_types.data_disk.DataDisk.load_by_value>`.
 
         See also
@@ -257,10 +278,12 @@ class DataBase():
         self.check_loaded()
         kw_keys = self.get_kw_keys(*keys, **kw_keys)
         keyring = self.loaded.get_keyring_by_index(by_day=by_day, **kw_keys)
-        return self.view(keyring=keyring)
+        return self.view(keyring=keyring, stack=stack, order=order)
 
     def view_selected(self, scope: Union[str, Scope] = 'selected',
-                      keyring: Keyring = None, **keys: KeyLike) -> np.ndarray:
+                      stack: Union[str, bool] = None, order: List[str] = None,
+                      keyring: Keyring = None,
+                      **keys: KeyLike) -> Union[Array, Tuple[Array]]:
         """Returns a subset of loaded data.
 
         Subset to view is specified by a scope.
@@ -268,9 +291,8 @@ class DataBase():
 
         :param scope: Scope indicating the selection to take.
             If str, can be {'avail', 'loaded', 'selected', 'current'},
-            corresponding scope will then be taken.
-            It must have been created from the loaded scope.
-            Defaults to current selection.
+            corresponding scope will then be taken. It must have been
+            created from the loaded scope. Defaults to current selection.
         :param keyring: [opt] Keyring specifying further slicing of selection.
         :param keys: [opt] Keys specifying further slicing of selection.
             Take precedence over keyring.
@@ -287,48 +309,7 @@ class DataBase():
 
         scope_ = scope.copy()
         scope_.slice(keyring, int2list=False, **keys)
-        return self.view(keyring=scope_.parent_keyring)
-
-    def view_ordered(self, order: List[str],
-                     keyring: Keyring = None, **keys: KeyLike) -> np.ndarray:
-        """Returns a reordered subset of data.
-
-        Keys act on loaded scope.
-        The ranks of the array are rearranged to follow
-        the specified coordinates order.
-
-        :param order: List of dimensions names in required order.
-            Either two dimensions (for swapping them)
-            or all of them should be specified.
-        :param keyring: [opt] Keyring specifying parts of dimensions to take.
-        :param keys: [opt] Keys specifying parts of dimensions to take.
-            Take precedence over keyring.
-
-        Examples
-        --------
-        >>> print(db.coords)
-        ['time', 'lat', 'lon']
-        >>> print(db.shape)
-        [12, 300, 500]
-        >>> a = db.view_orderd(['lon', 'lat'], time=[1])
-        ... print(a.shape)
-        [1, 500, 300]
-
-        See also
-        --------
-        Accessor.reorder: The underlying function.
-        view: For details on subsetting data (without reordering).
-        """
-        self.check_loaded()
-
-        keyring = Keyring.get_default(keyring, **keys, variables=self.loaded.var)
-        keyring.make_full(self.dims)
-        keyring.make_total()
-        keyring.sort_by(self.dims)
-
-        log.debug('Taking keys in data: %s', keyring.print())
-        array = self.acs.take(keyring, self.data)
-        return self.acs.reorder(keyring, array, order)
+        return self.view(keyring=scope_.parent_keyring, stack=stack, order=order)
 
     def iter_slices(self, coord: str, size: int = 12,
                     key: KeyLike = None) -> List[KeyLike]:
@@ -368,29 +349,18 @@ class DataBase():
         """Number of coordinates."""
         return len(self.coords)
 
-    @property
-    def shape(self) -> List[int]:
-        """Shape of the data from current scope.
-
-        Scope is loaded if not empty, available otherwise.
-        """
-        return self.scope.shape
-
     def get_limits(self, *coords: str,
                    scope: Union[str, Scope] = 'current',
                    keyring: Keyring = None, **keys: KeyLike) -> List[float]:
         """Return limits of coordinates.
 
         Min and max values for specified coordinates.
-        Scope is loaded if not empty, available otherwise.
 
         :param coords: [opt] Coordinates name.
-            If None, defaults to all coordinates, in the order
-            of data.
+            If None, defaults to all coordinates in order.
         :param scope: [opt] Scope to use. Default is current.
         :param keyring: [opt] Subset coordinates.
-        :param keys: [opt] Subset coordinates.
-            Take precedence over keyring.
+        :param keys: [opt] Subset coordinates. Take precedence over keyring.
 
         :returns: Min and max of each coordinate. Flattened.
 
@@ -413,12 +383,10 @@ class DataBase():
         Return first and last value of specified coordinates.
 
         :param coords: [opt] Coordinates name.
-            If None, defaults to all coordinates, in the order
-            of data.
+            If None, defaults to all coordinates, in the order.
         :param scope: [opt] Scope to use. Default is current.
         :param keyring: [opt] Subset coordinates.
-        :param keys: [opt] Subset coordinates.
-            Take precedence over keyring.
+        :param keys: [opt] Subset coordinates. Take precedence over keyring.
 
         :returns: First and last values of each coordinate.
 
@@ -436,19 +404,15 @@ class DataBase():
     def get_kw_keys(self, *keys: KeyLike, **kw_keys: KeyLike) -> Dict[str, KeyLike]:
         """Make keyword keys when asking for coordinates parts.
 
-        From a mix of positional and keyword argument,
-        make a list of keywords.
+        From a mix of positional and keyword argument, make a list of keywords.
         Keywords arguments take precedence over positional arguments.
         Positional argument shall be ordered as the coordinates
         are ordered in data.
 
-        :param keys: [opt]
-        :param kw_keys: [opt]
-
         Examples
         --------
-        >>> print( db.get_kw_keys([0, 1], lat=slice(0, 10)) )
-        {'time': [0, 1], 'lat': slice(0, 10)}
+        >>> print( db.get_kw_keys('SST', [0, 1], lat=slice(0, 10)) )
+        {'var': 'SST', 'time': [0, 1], 'lat': slice(0, 10)}
         """
         for i, key in enumerate(keys):
             name = self.dims[i]
@@ -458,15 +422,18 @@ class DataBase():
 
     def get_subscope(self, scope: Union[str, Scope] = 'avail',
                      keyring: Keyring = None, int2list: bool = True,
-                     name: str = None,
-                     **keys: KeyLike) -> Scope:
+                     name: str = None, **keys: KeyLike) -> Scope:
         """Return subset of scope.
 
         :param scope: [opt] Scope to subset.
             If str, can be {'avail', 'loaded', 'selected', 'current'},
             corresponding scope of data will then be taken.
-        :param keyring: [opt]
+        :param keyring: [opt] Act on `scope`.
         :param keys: [opt]
+        :param name: [opt] Name of new scope. If None will take
+            name of input scope.
+        :param int2list: If True, integer keys are turned to list.
+            This avoids squeezing dimensions. Default to True.
 
         :returns: Copy of input scope, sliced with specified keys.
         """
@@ -480,16 +447,17 @@ class DataBase():
         return subscope
 
     def get_subscope_by_value(self, scope: Union[str, Scope] = 'avail',
-                              int2list: bool = True,
-                              name: str = None,
+                              int2list: bool = True, name: str = None,
                               by_day: bool = False,
                               **keys: KeyLikeValue) -> Scope:
         """Return subset of scope.
 
-        :param bool: Use `subset_by_day` for Time dimension rather than `subset`.
-            Default to False.
-        :param kw_keys: [opt] Argument name is dimension name for value selection,
-            or dimension name appended with `_idx` for index selection.
+        Arguments work similarly as :func:`DataDisk.load_by_value
+        <tomate.db_types.data_disk.DataDisk.load_by_value>`.
+
+        See also
+        --------
+        get_subscope
         """
         scope = self.get_scope(scope)
         keyring = scope.get_keyring_by_index(by_day=by_day, **keys)
@@ -501,10 +469,10 @@ class DataBase():
 
         :param scope: [opt] Scope to select from.
             If str, can be {'avail', 'loaded', 'selected', 'current'},
-            corresponding scope will then be taken.
-            Default to current scope (loaded if data has been loaded,
-            avail otherwise).
+            corresponding scope will then be taken. Default to current scope
+            (loaded if data has been loaded, avail otherwise).
         :param keyring: [opt] Keyring specifying parts of dimensions to take.
+            Act on the specified scope.
         :param keys: [opt] Keys specifying parts of dimensions to take.
             Act on the specified scope. Take precedence over `keyring`.
 
@@ -524,13 +492,8 @@ class DataBase():
                         by_day: bool = False, **keys: KeyLike):
         """Set selection scope by value.
 
-        :param scope: [opt] Scope to select from. See `select()`.
-        :param by_day: If True, find indices prioritising dates.
-            See :ref:`Some examples of coordinates subclasses` for details.
-        :param keys: [opt] Keys specifying parts of dimensions to take
-            by value. Take precedence over `keyring`. Act on specified scope.
-            See 'kw_keys' argument of :func:`DataDisk.load_by_value
-            <tomate.db_types.data_disk.DataDisk.load_by_value>` for details.
+        Arguments work similarly as :func:`DataDisk.load_by_value
+        <tomate.db_types.data_disk.DataDisk.load_by_value>`.
 
         Examples
         --------
@@ -543,26 +506,25 @@ class DataBase():
         get_subscope_by_value: select_by_value() is a wrapper around
             get_subscope_by_value().
         """
-        self.selected = self.get_subscope_by_value(scope, int2list=True,
+        self.selected = self.get_subscope_by_value(scope, int2list=False,
                                                    by_day=by_day,
                                                    name='selected', **keys)
 
     def add_to_selection(self, scope: Union[str, Scope] = 'avail',
                          keyring: Keyring = None, **keys: KeyLike):
-        """Add to selection.
+        """Expand 'selected' scope.
 
-        Keys act on the parent scope of selection.
-        Keys are always sorted in increasing order
-
-        :param Scope: [opt] If nothing was selected before, select keys from this scope.
+        :param Scope: [opt] If nothing was selected before, select keys from
+             this scope.
         :param keyring: [opt]
-        :param keys: [opt]
+        :param keys: [opt] Keys act on the parent scope of selection.
         """
         scope = self.selected
         if scope.is_empty():
             self.select(scope, keyring=keyring, **keys)
         else:
-            keyring = Keyring.get_default(keyring, **keys, variables=self.avail.var)
+            keyring = Keyring.get_default(keyring, **keys, dims=self.avail.dims)
+            keyring.make_str_idx(**scope.parent_scope.dims)
             keyring.set_shape(scope.parent_scope.dims)
             keyring = keyring + scope.parent_keyring
             keyring.sort_keys()
@@ -572,107 +534,144 @@ class DataBase():
     def slice_data(self, keyring: Keyring = None, **keys: KeyLike):
         """Slice loaded data.
 
-        Keys act on loaded scope.
+        :param keys: Keys act on loaded scope.
         """
         self.check_loaded()
-        keyring = Keyring.get_default(keyring, **keys)
-        keyring.make_int_list()
-        self.loaded = self.get_subscope('loaded', keyring)
-        self.data = self.view(keyring=keyring)
+        self.loaded.slice(keyring=keyring, **keys)
 
-    def unload_data(self):
-        """Remove data."""
-        self.data = None
+        for var in self.variables.values():
+            if var.name not in self.loaded:
+                var.unload()
+            else:
+                var.data = var.view(keyring=keyring, **keys)
+
+    def unload(self):
+        """Remove all data."""
         self.loaded.empty()
+        for var in self.variables.values():
+            var.unload()
 
-    def allocate(self, shape: List[int]) -> np.ndarray:
-        """Allocate data array.
-
-        :param shape: Shape of the array to allocate.
-        """
-        log.info("Allocating numpy array of shape %s", shape)
-        return self.acs.allocate(shape)
-
-    def self_allocate(self, shape: List[int]):
-        """Allocate data array for itself.
-
-        :param shape: Shape of the array to allocate.
-        """
-        self.data = self.allocate(shape)
-
-    def set_data(self, variable: str, data: np.ndarray,
-                 keyring: Keyring = None):
+    def set_data(self, variable: str, data: Array,
+                 keyring: Keyring = None, **keys: KeyLike):
         """Set the data for a single variable.
 
-        :param var: Variable to set the data to.
-        :param data: Array of the correct shape for currently
-            selected coordinates. Has no axis for variables.
-        :param keyring: [opt] If no data is loaded, loaded scope
-            is fetched from available scope with this keyring.
-            'var' key has no effect.
+        Wrapper around :func:`Variable.set_data
+        <tomate.variable_base.Variable.set_data>`.
 
-        :raises KeyError: If the variable is not in available scope.
-        :raises IndexError: If the data has not the right dimension.
-        :raises ValueError: If the data is not of the shape of loaded scope.
+        :raises KeyError: If the variable was not created.
         """
-        def check_shape(data):
-            if self.acs.shape(data)[1:] != self.shape[1:]:
-                raise ValueError("data of wrong shape ({}, expected {})"
-                                 .format(self.acs.shape(data)[1:], self.shape[1:]))
+        if variable not in self.avail or variable not in self.variables:
+            raise KeyError(f"{variable} was not created. Use add_variable")
+        self.variables[variable].set_data(data, keyring, **keys)
+
+    def add_variable(self, variable: str, dims: List[str] = None,
+                     var_class: Type = None, datatype: Any = None,
+                     attrs: Dict[str, any] = None) -> Variable:
+        """Add a new variable.
+
+        Create variable object, and add variable to available scope.
+
+        If `dims`, `var_class`, or `datatype` are not specified,
+        the value in the VI for the pair `variable` / `_dims`,
+        `_var_class`, `_datatype` are used, if not in the VI, the
+        value in the VI without the leading underscores, and if not
+        in the VI either, the default will be used (see below).
+
+        :param variable: Name of variable
+        :param dims: [opt] List of dimensions this variable vary along.
+            If not specified or in VI, Tomate will try to guess
+            from information in Filegroups.
+            If it can't, defaults to all coordinates available.
+        :param var_class: [opt] Variable subclass to use.
+            Default to :class:`Variable<variables.base.Variable>`.
+        :param datatype: [opt] Datatype for the array.
+            Default to None.
+        :param attrs: [opt] Attributes to put in the VI.
+        """
+        if variable in self.variables:
+            log.warning('%s already in variables, it will be overwritten.')
+
+        if dims is None:
+            dims = self.vi.get_attribute_param(variable, 'dims', None)
+        if dims is None:
+            if hasattr(self, 'var_disk') and variable in self.var_disk:
+                dims = guess_dimensions(self, variable)
+        if dims is None:
+            dims = self.coords
+
+        if var_class is None:
+            var_class = self.vi.get_attribute_param(variable, 'var_class', Variable)
+        if var_class is None:
+            var_class = self.vi.get_attribute_param('_all', 'var_class', Variable)
+
+        if attrs is not None:
+            self.vi.set_attributes(variable, **attrs)
+
+        db_var = var_class(variable, dims, self)
 
         if variable not in self.avail:
-            raise KeyError(f"{variable} is not in avail scope. Use add_variable.")
-        if self.acs.ndim(data) != self.ncoord:
-            raise IndexError("data of wrong dimension ({}, expected {})"
-                             .format(data.ndim, self.ncoord))
-
-        data = np.expand_dims(data, 0)
-
-        # No data is loaded
-        if self.loaded.is_empty():
-            self.loaded = self.get_subscope('avail', keyring=keyring, var=variable)
-            check_shape(data)
-            self.data = data
-
-        # Variable is already loaded
-        elif variable in self.loaded.var:
-            check_shape(data)
-            self[variable][:] = data[0]
-
-        # Variable is not loaded, others are
-        else:
-            self.loaded.var.append(variable)
-            check_shape(data)
-            self.data = self.acs.concatenate((self.data, data), axis=0)
-
-    def add_variable(self, variable: str, data: np.ndarray = None,
-                     keyring: Keyring = None, **attrs: Any):
-        """Add new variable.
-
-        Add variable to available scope,
-        and its attributes to the VI.
-        If present, add data to loaded data.
-
-        :param variable: Variable to add.
-        :param data: [opt] Corresponding data to add.
-            Its shape must match that of the loaded scope.
-        :param keyring: [opt] If there is data to set and no data is loaded,
-            loaded scope is fetched from available scope with this keyring.
-            'var' key has no effect.
-        :param attrs: [opt] Variable attributes.
-            Passed to VariablesInfo.add_variable
-        """
-        if variable in self.avail:
-            log.warning('%s already in avail scope, it will be overwritten.', variable)
-        else:
             self.avail.var.append(variable)
-        if data is not None:
-            self.set_data(variable, data, keyring=keyring)
-        self.vi.set_attrs(variable, **attrs)
 
-    def remove_loaded_variable(self, variables: Union[str, List[str]]):
-        """Remove variable from data."""
+        if datatype is None:
+            datatype = self.vi.get_attribute_param(variable, 'datatype', None)
+        if datatype is None:
+            datatype = self.vi.get_attribute_param('_all', 'datatype', None)
+        db_var.datatype = datatype
+
+        self.variables[variable] = db_var
+        return db_var
+
+    def create_variables(self, disk=False, replace=False):
+        """Create variables objects."""
+        for var in self.avail['var']:
+            if disk:
+                self.var_disk.add(var)
+            if var not in self.variables or replace:
+                self.add_variable(var)
+
+    def remove_loaded_variables(self, variables: Union[str, List[str]]):
+        """Remove variables from loaded scope."""
         if isinstance(variables, str):
             variables = [variables]
         keep = [v for v in self.loaded if v not in variables]
         self.slice_data(var=keep)
+
+
+def guess_dimensions(db: 'DataDisk', var: str):
+    """Guess variables dimensions from filegroups."""
+
+    try:
+        krg = Keyring(var=var)
+        krg.make_str_idx(var=db.avail.var)
+        dims_fg = []
+        for fg in db.filegroups:
+            cmd = fg.get_fg_keyrings(krg)
+            if cmd is not None:
+                key = cmd.infile['var']
+                key.make_list_int()
+                dims = key.apply(fg.cs['var'].dimensions)
+                dims = [d for d in dims if d in db.dims]
+                dims_fg.append(dims)
+    except Exception:
+        return None
+
+    if len(dims_fg) > 0:
+        lengths = [len(z) for z in dims_fg]
+        dims = list(dims_fg[lengths.index(max(lengths))])
+    else:
+        return None
+
+    log.debug('Guessed dimensions %s for variable %s', dims, var)
+
+    warn = False
+    for d in db.coords:
+        if d not in dims:
+            for fg in db.filegroups:
+                if fg.cs[d].shared:
+                    warn = True
+
+    if warn:
+        log.warning(("'%s' dimension is shared, but not in guess (%s), "
+                     "something might be wrong"), d, dims)
+
+    return dims
