@@ -6,8 +6,11 @@
 # at the root of this project. © 2020 Clément HAËCK
 
 
+from typing import List
+
 import matplotlib.image
 
+from tomate.coordinates.time import Time
 from tomate.custom_types import KeyLikeInt
 from tomate.db_types.plotting.image_abc import PlotObjectImageABC
 
@@ -31,11 +34,22 @@ class PlotObjectImage(PlotObjectImageABC):
 
     def create_plot(self):
         image = self.get_data()
-        extent = self.scope.get_extent(*self.axes[:2])
-        self.object = self.ax.imshow(image, extent=extent, **self.kwargs)
+        self.object = self.ax.imshow(image, extent=self._get_extent(),
+                                     **self.kwargs)
 
     def update_plot(self, **keys: KeyLikeInt):
-        self.up_scope(**keys)
+        self.update_scope(**keys)
         image = self.get_data()
         self.image.set_data(image)
-        self.image.set_extent(self.scope.get_extent(*self.axes[:2]))
+        self.image.set_extent(self._get_extent())
+
+    def _get_extent(self) -> List[float]:
+        extent = []
+        for name in self.axes[:2]:
+            dim = self.scope[name]
+            if isinstance(dim, Time):
+                extent += dim.change_units_other(dim[[0, -1]], dim.units,
+                                                 'days since 01-01-01').tolist()
+            else:
+                extent += dim.get_extent()
+        return extent
