@@ -18,8 +18,8 @@ class PlotObjectImageABC(PlotObjectABC):
     DIM = 2
 
     def _get_data(self) -> Array:
-        image = self.db.view_ordered(self.axes[1::-1],
-                                     keyring=self.keyring)
+        image = self.db.view_selected(self.scope,
+                                      order=self.axes[1::-1])
         return image
 
     def find_axes(self, axes: List[str] = None) -> List[str]:
@@ -34,6 +34,8 @@ class PlotObjectImageABC(PlotObjectABC):
         if len(axes_) == 2:
             axes_.append(self.scope.var[0])
 
+        self.var_idx = 2
+
         return axes_
 
     def set_limits(self):
