@@ -1,10 +1,10 @@
 """Objects for indexing arrays."""
 
-from .key import Key, KeyVar
+from .key import Key, KeyValue
 from .keyring import Keyring
 
 __all__ = [
     'Key',
-    'KeyVar',
+    'KeyValue',
     'Keyring'
 ]
